# -*- coding: utf-8 -*-

# Copyright(C) 2014 Romain Bignon
#
# This file is part of weboob.
#
# weboob is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# weboob is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with weboob. If not, see <http://www.gnu.org/licenses/>.


import lxml.html as html

from weboob.browser.filters.standard import _Selector, _NO_DEFAULT, Filter, FilterError
from weboob.tools.html import html2text
from urlparse import urljoin


__all__ = ['CSS', 'XPath', 'XPathNotFound', 'AttributeNotFound',
           'Attr', 'Link', 'CleanHTML']


from weboob.browser.filters.html import XPathNotFound as _XPathNotFound
class XPathNotFound(_XPathNotFound):
    pass


from weboob.browser.filters.html import AttributeNotFound as _AttributeNotFound
class AttributeNotFound(_AttributeNotFound):
    pass


from weboob.browser.filters.html import CSS as _CSS
class CSS(_CSS):
    def select(self, selector, item):
        ret = item.cssselect(selector)
        if isinstance(ret, list):
            for el in ret:
                if isinstance(el, html.HtmlElement):
                    self.highlight_el(el, item)

        return ret


from weboob.browser.filters.html import XPath as _XPath
class XPath(_XPath):
    pass


from weboob.browser.filters.html import Attr as _Attr
class Attr(_Attr):
    def __init__(self, selector, attr, default=_NO_DEFAULT):
        super(_Attr, self).__init__(selector, default=default)
        self.attr = attr

    def filter(self, el):
        try:
            return u'%s' % el[0].attrib[self.attr]
        except IndexError:
            return self.default_or_raise(XPathNotFound('Unable to find tag %s' % self.selector))
        except KeyError:
            return self.default_or_raise(AttributeNotFound('Element %s does not have attribute %s' % (el[0], self.attr)))


from weboob.browser.filters.html import Link as _Link
class Link(_Link):
    """
    Get the link uri of an element.

    If the <a> tag is not found, an exception IndexError is raised.
    """

    def __init__(self, selector=None, default=_NO_DEFAULT):
        super(_Link, self).__init__(selector, 'href', default=default)


class AbsoluteLink(Link):
    def __call__(self, item):
        ret = super(AbsoluteLink, self).__call__(item)
        if ret:
            ret = urljoin(item.page.url, ret)
        return ret


from weboob.browser.filters.html import CleanHTML as _CleanHTML
class CleanHTML(_CleanHTML):
    def filter(self, txt):
        if isinstance(txt, (tuple, list)):
            return u' '.join([self.clean(item) for item in txt])
        return self.clean(txt)

    @classmethod
    def clean(cls, txt):
        if not isinstance(txt, basestring):
            txt = html.tostring(txt, encoding=unicode)
        return html2text(txt)

